/**
 *      partie_gtk.c
 *      Contient les fonctions pour la gestion du jeu en GTK
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>


/* Inclusions du programme */
#include "../../typedef.h"
#include "gtkdef.h"
#include "../../common.h"
#include "../../jeu/gestion_liste.h"
#include "../../jeu/damier.h"
#include "../../jeu/coups.h"
#include "../../jeu/pieces.h"
#include "../../jeu/gestion_sauvegarde.h"
#include "../../jeu/jeu.h"
#include "../../ia/choix_coup.h"

#include "interface_gtk.h"
#include "partie_gtk.h"


/* Initialise la structure de la partie
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de jeu
 * Sortie: (en paramètre) La structure initialisée
 */
void init_jeu(Tjeu* jeu)
{
	*jeu = (Tjeu)malloc(sizeof(_Tjeu));
	init_liste(&((*jeu)->coup));
	init_liste(&((*jeu)->coups_possibles));
	init_liste(&((*jeu)->listecoups));
	init_damier((*jeu)->damier);
	(*jeu)->tour = 'B';
	(*jeu)->partance = 'B';
	(*jeu)->j_blanc = JOUEUR_HUMAIN;
	(*jeu)->j_noir = JOUEUR_HUMAIN;
	(*jeu)->finpartie = 0;
	(*jeu)->bouton_valider = NULL;
	(*jeu)->liste_radio = NULL;
	(*jeu)->cases_selection = NULL;
	(*jeu)->statusbar = NULL;
	(*jeu)->statusbar_context = 0;
	generateur_coups((*jeu)->damier,(*jeu)->tour,&((*jeu)->coups_possibles));
}


/* Libère l'espace occupé par la structure de la partie
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de jeu
 * Sortie: (en paramètre) La structure vidée
 */
void free_jeu(Tjeu* jeu)
{
	int x,y;

	free_liste(&((*jeu)->listecoups));
	free_liste(&((*jeu)->coups_possibles));
	free_liste(&((*jeu)->coup));
	gtk_list_store_clear((*jeu)->cases_selection);

	for (y=0; y<tailleDamier; y++)
	{
		for (x=0; x<tailleDamier; x++)
		{
			gtk_widget_destroy((*jeu)->labels[x][y]);
			gtk_widget_destroy((*jeu)->cases[x][y]);
		}
	}

	gtk_widget_destroy((*jeu)->statusbar);
	gtk_widget_destroy((*jeu)->bouton_valider);

	free(*jeu);

	*jeu = NULL;
}


/* Fonction de callback pour annuler les cases sélectionnés
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (en paramètre) La structure de la partie
 */
void annuler_coup(GtkWidget *widget, gpointer data)
{
	Tjeu jeu = (Tjeu)data;

	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);

	(void)widget;
}


/* Fonction de callback pour valider le coup
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (en paramètre) La structure de la partie
 */
void valider_coup(GtkWidget *widget, gpointer data)
{
	Tjeu jeu = (Tjeu)data;

	/* Jeu Humain */
	if ((jeu->tour=='B' && jeu->j_blanc == JOUEUR_HUMAIN) || ((jeu->tour=='N' && jeu->j_noir == JOUEUR_HUMAIN)))
	{
		if (jeu->coup!=NULL)
		{
			if (verification_coup(*(jeu->coup), jeu->coups_possibles))
			{
				execute_coup(jeu->damier, *(jeu->coup));
				sauve_coup(&(jeu->listecoups), *(jeu->coup));
				jeu->finpartie = fin_partie(jeu->damier, *(jeu->coup));
				jeu->tour = adversaire(jeu->tour);
				free_liste(&(jeu->coups_possibles));
				generateur_coups(jeu->damier, jeu->tour, &(jeu->coups_possibles));
				refresh_table(jeu);
			}
			else
			{
				GtkWidget *p_dialog;

				p_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "Coup non permis\nEst ce que c'est bien le plus long ?");
				gtk_dialog_run (GTK_DIALOG (p_dialog));
				gtk_widget_destroy (p_dialog);
			}
		}

		free_liste(&(jeu->coup));
		gtk_list_store_clear(jeu->cases_selection);

		sauve_liste_coup(jeu->listecoups, FICHIER_SAUVEGARDE);
		update_status(jeu);

		/* Au tour de l'IA */
		if ((jeu->tour=='B' && jeu->j_blanc == JOUEUR_IA) || ((jeu->tour=='N' && jeu->j_noir == JOUEUR_IA)))
		{
			gtk_widget_activate(jeu->bouton_valider);
		}

	}
	else /* Au tour de l'IA */
	{
		free_liste(&(jeu->coup));
		gtk_list_store_clear(jeu->cases_selection);

		ia_joue(widget, data);
		update_status(jeu);

		gtk_widget_activate(jeu->bouton_valider);
	}

	(void)widget;
}


/* Fonction de callback pour faire jouer le coup par l'IA
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (en paramètre) La structure de la partie
 */
void ia_joue(GtkWidget *p_widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;
	Tcoup* coup_ia;
	int valeur = 0;

	coup_ia = choix_coup(jeu->tour, jeu->damier, jeu->coups_possibles, &valeur);

	if (coup_ia != NULL)
	{
		execute_coup(jeu->damier, *coup_ia);
		sauve_coup(&(jeu->listecoups), *coup_ia);
		jeu->finpartie = fin_partie(jeu->damier, *coup_ia);
		refresh_table(jeu);
		jeu->tour = adversaire(jeu->tour);
		free_liste(&(jeu->coups_possibles));
		free_liste(&(jeu->coup));
		gtk_list_store_clear(jeu->cases_selection);
		generateur_coups(jeu->damier, jeu->tour, &(jeu->coups_possibles));
	}
	else
		jeu->finpartie = 1;

	sauve_liste_coup(jeu->listecoups, FICHIER_SAUVEGARDE);
	update_status(jeu);

	if ((jeu->tour=='B' && jeu->j_blanc == JOUEUR_IA) || ((jeu->tour=='N' && jeu->j_noir == JOUEUR_IA)))
		gtk_widget_activate(jeu->bouton_valider);

	(void)p_widget;
}


/* Fonction de callback pour faire passer son tour au joueur
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (en paramètre) La structure de la partie
 */
void passer_tour(GtkWidget *p_widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	jeu->tour = adversaire(jeu->tour);
	free_liste(&(jeu->coups_possibles));
	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);
	generateur_coups(jeu->damier, jeu->tour, &(jeu->coups_possibles));

	update_status(jeu);

	if ((jeu->tour=='B' && jeu->j_blanc == JOUEUR_IA) || ((jeu->tour=='N' && jeu->j_noir == JOUEUR_IA)))
		gtk_widget_activate(jeu->bouton_valider);


	(void)p_widget;
}


/* Fonction de callback pour réinitialiser le damier
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void reinit_table(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	init_damier(jeu->damier);
	jeu->tour = jeu->partance;

	free_liste(&(jeu->listecoups));
	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);
	generateur_coups(jeu->damier, jeu->tour, &(jeu->coups_possibles));

	refresh_table(jeu);
	update_status(jeu);

	(void)widget;
}


/* Fonction de callback pour lancer la partie humain-humain
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void start_new_humain_vs_humain(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	jeu->j_blanc = JOUEUR_HUMAIN;
	jeu->j_noir = JOUEUR_HUMAIN;

	reinit_table(widget, user_data);
	free_liste(&(jeu->listecoups));
	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);

	update_status(jeu);

	(void)widget;
}


/* Fonction de callback pour lancer la partie humain-ia
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void start_new_humain_vs_ia (GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	demander_camp(jeu);

	reinit_table(widget, user_data);
	free_liste(&(jeu->listecoups));
	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);

	update_status(jeu);

	(void)widget;
	(void)user_data;
}


/* Fonction de callback pour lancer la partie ia-ia
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void start_new_ia_vs_ia (GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	jeu->j_blanc = JOUEUR_IA;
	jeu->j_noir = JOUEUR_IA;

	reinit_table(widget, user_data);
	free_liste(&(jeu->listecoups));
	free_liste(&(jeu->coup));
	gtk_list_store_clear(jeu->cases_selection);

	update_status(jeu);

	(void)widget;
}


/* Fonction de callback pour sauvegarder la partie
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void sauve_partie(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;
	GtkWidget *p_dialog = NULL;
	GtkFileFilter *filtre;


	p_dialog = gtk_file_chooser_dialog_new ("Ouvrir un fichier", NULL,GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);

	filtre = gtk_file_filter_new() ;
	gtk_file_filter_set_name (filtre, "Fichiers de sauvegarde (*.sav)") ;
	gtk_file_filter_add_pattern(filtre, "*.sav") ;
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (p_dialog), filtre) ;
	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (p_dialog), filtre) ;
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER (p_dialog),TRUE);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER (p_dialog), FICHIER_SAUVEGARDE);

	if (gtk_dialog_run (GTK_DIALOG (p_dialog)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *file_name = NULL;

		file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (p_dialog));
		sauve_liste_coup(jeu->listecoups,file_name);
		g_free (file_name), file_name = NULL;

	}
	gtk_widget_destroy (p_dialog);

	/* Parametre inutilise */
	(void)widget;
}


/* Fonction de callback pour charger la partie
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void charge_partie(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;
	GtkWidget *p_dialog = NULL;
	GtkFileFilter *filtre;

	p_dialog = gtk_file_chooser_dialog_new ("Ouvrir un fichier", NULL,GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);

	filtre = gtk_file_filter_new() ;
	gtk_file_filter_set_name (filtre, "Fichiers de sauvegarde (*.sav)") ;
	gtk_file_filter_add_pattern(filtre, "*.sav") ;
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (p_dialog), filtre) ;
	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (p_dialog), filtre) ;
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER (p_dialog), FICHIER_SAUVEGARDE);

	if (gtk_dialog_run (GTK_DIALOG (p_dialog)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *file_name = NULL;

		file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (p_dialog));
		free_liste(&(jeu->listecoups));
		charge_liste_coup(&(jeu->listecoups),file_name);
		g_free (file_name), file_name = NULL;

		lit_sauvegarde_unlock(jeu->listecoups,jeu->damier);
		refresh_table(jeu);
		inverse_liste(&(jeu->listecoups));	/* Oui, la sauvegarde est à l'envers parce que c'est plus facile... */

		jeu->tour = 'B';
		free_liste(&(jeu->coups_possibles));
		generateur_coups(jeu->damier,jeu->tour,&(jeu->coups_possibles));
	}
	gtk_widget_destroy (p_dialog);

	/* Parametre inutilise */
	(void)widget;
}
