/**
 *      gestion_liste.c
 *      Contient les fonctions de gestion de liste chainée
 *
 *      Mathieu Pellegrin - 2009
 *
 **/


/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "gestion_liste.h"


/* Initialisation de la liste vide
 * Auteur: Mathieu Pellegrin
 * Entrée: L'adresse du pointeur de tête
 * Sortie: (en paramètres) la liste initialisée
 */
void init_liste(Tliste *l)
{
	*l = NULL;
}


/* Test de vacuité
 * Auteur: Mathieu Pellegrin
 * Entrée: Le pointeur de tête
 * Sortie: 1 si vide et 0 sinon
 */
int liste_vide(Tliste l)
{
	return (l == NULL);
}


/* Comparaison d'éléments de la liste (selon le type de la liste)
 * Auteur: Mathieu Pellegrin
 * Entrée: 2 éléments
 * Sortie: 1 si égalité et 0 si différence
 */
int compare_elements(Tcoup* t1, Tcoup* t2)
{
	if (t1!=NULL && t2!=NULL)
	{
		return  (
		    ((t1->depart).x == (t2->depart).x && (t1->depart).y == t2->depart.y) &&
		    ((t1->arrivee).x == (t2->arrivee).x && (t1->arrivee).y == (t2->arrivee).y)
		);
	}
	else
		return (t1 == t2);
}


/* Test d'appartenance
 * Auteur: Mathieu Pellegrin
 * Entrée: Le pointeur de tête, l'élément à rechercher
 * Sortie: 1 si présent et 0 sinon
 */
int element_existe(Tliste l, Tcoup e)
{
	while (l!=NULL && !compare_elements(l,&e))
	{
		l = l->suiv;
	}
	if (l==NULL)
		return 0;
	else
		return 1;
}


/* Calcule la longueur de la liste
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste
 * Sortie: La longueur de la liste
 */
int longueur_liste(Tliste l)
{
	int i = 0;
	Tcoup* tmp = l;

	while (tmp != NULL)
	{
		i++;
		tmp = tmp->suiv;
	}

	return i;
}


/* Inverse le chainage de la liste
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste
 * Sortie: (paramètres) La liste inversée
 */
void inverse_liste(Tliste *l)
{
	Tliste prec = NULL;
	Tliste tmp = *l;
	Tliste suiv;
	if (tmp!=NULL)
		suiv = tmp->suiv;

	while (tmp!=NULL)
	{
		tmp->suiv = prec;
		prec = tmp;
		tmp = suiv;
		if (suiv!=NULL)
			suiv = suiv->suiv;
	}

	*l = prec;
}


/* Copie d'un maillon
 * Auteur: Mathieu Pellegrin
 * Entrée: Le maillon à copier
 * Sortie: (paramètre) Le maillon copié
 */
void copie_maillon(Tcoup* original, Tcoup** copie)
{
	*copie = (Tcoup*)malloc(sizeof(Tcoup));

	if (original == NULL)
	{
		*copie = NULL;
	}
	else
	{
		(*copie)->depart.x = original->depart.x;
		(*copie)->depart.y = original->depart.y;
		(*copie)->arrivee.x = original->arrivee.x;
		(*copie)->arrivee.y = original->arrivee.y;
		(*copie)->suiv = original->suiv;
		(*copie)->multiple = original->multiple;
	}

	#if DEBUG_LVL>=3
	printf("COPIE copie_maillon %p %d %d %d %d %p %p\n",(void*)original,original->depart.x,original->depart.y,original->arrivee.x,original->arrivee.y,(void*)original->multiple,(void*)original->suiv);
	printf("ALLOC copie_maillon %p %d %d %d %d %p %p\n",(void*)(*copie),(*copie)->depart.x,(*copie)->depart.y,(*copie)->arrivee.x,(*copie)->arrivee.y,(void*)(*copie)->multiple,(void*)(*copie)->suiv);
	#endif
}


/* Ajout d'un nouvel élément dans la liste
 * Auteur: Mathieu Pellegrin
 * Entrée: L'adresse du pointeur de tête, l'élément à ajouter
 * Sortie: Rien
 */
void ajoute_element(Tliste *l, Tcoup e)
{
	Tliste tmp = (Tliste)malloc(sizeof(Tcoup));
	tmp->depart.x = e.depart.x;
	tmp->depart.y = e.depart.y;
	tmp->arrivee.x = e.arrivee.x;
	tmp->arrivee.y = e.arrivee.y;
	tmp->suiv = *l;
	tmp->multiple = e.multiple;
	*l = tmp;

	#if DEBUG_LVL>=3
	printf("ALLOC ajoute_element %p %d %d %d %d %p %p\n",(void*)tmp,tmp->depart.x,tmp->depart.y,tmp->arrivee.x,tmp->arrivee.y,(void*)tmp->multiple,(void*)tmp->suiv);
	#endif
}


/* Suppression d'un élément dans la liste
 * Auteur: Mathieu Pellegrin
 * Entrée: L'adresse du pointeur de tête, l'élément à supprimer
 * Sortie: Rien
 */
void supprime_element(Tliste *l, Tcoup e)
{
	Tcoup* tmp = NULL;
	Tcoup* tmp1 = NULL;
	Tcoup* tmp2 = NULL;
	int stop = 0;

	if (l!=NULL && *l!=NULL)
	{
		/* Cas particulier pour la tête (toujours) */
		if (compare_elements(*l,&e))
		{
			tmp = *l;
			*l = (*l)->suiv;
			free_maillon(tmp);
			stop = 1;
		}

		/* Recherche des éléments */
		if (!stop)
		{
			tmp1 = *l;
			tmp2 = (*l)->suiv;
		}
		/* Tant que non trouvé */
		while (!stop && tmp2!=NULL)
		{
			if(compare_elements(tmp2,&e))
			{
				tmp = tmp2;
				tmp1->suiv = tmp2->suiv;
				free_maillon(tmp);
				stop = 1;
			}
		}
	}
}


/* Désallocation d'un maillon (ses multiples et tous les suivants de tous ses multiples)
 * Auteur: Mathieu Pellegrin
 * Entrée: L'adresse du maillon
 * Sortie: Rien
 */
void free_maillon(Tcoup *m)
{
	Tcoup* tmp;
	Tcoup *mult = m->multiple;

	#if DEBUG_LVL>=3
	printf("FREE free_maillon %p %d %d %d %d %p %p\n",(void*)m,m->depart.x,m->depart.y,m->arrivee.x,m->arrivee.y,(void*)m->multiple,(void*)m->suiv);
	#endif

	free(m);

	while (mult!=NULL)
	{
		/*if (mult->suiv!=NULL)
			printf("BUG %p %d %d %d %d %p %p\n",(void*)mult,mult->depart.x,mult->depart.y,mult->arrivee.x,mult->arrivee.y,(void*)mult->multiple,(void*)mult->suiv);*/
		tmp = mult;
		mult = mult->suiv;
		free_maillon(tmp);
	}
}


/* Désallocation de la liste complète
 * Auteur: Mathieu Pellegrin
 * Entrée: L'adresse du pointeur de tête
 * Sortie: Rien
 */
void free_liste(Tliste *l)
{
	Tcoup* tmp;

	while (*l!=NULL)
	{
		tmp = *l;
		*l = (*l)->suiv;
		free_maillon(tmp);
	}
	*l = NULL;
}


/* Affichage des éléments de l'ensemble
 * Auteur: Mathieu Pellegrin
 * Entrée: Le pointeur de tête
 * Sortie: (écran) l'ensemble de la liste
 */
void affiche_liste(Tliste l)
{
	Tliste mult = NULL;
	while (l!=NULL)
	{
		printf("Depart: %d;%d , Arrivee %d;%d",(l->depart).x,(l->depart).y,(l->arrivee).x,(l->arrivee).y);
		mult = l->multiple;
		while (mult!=NULL)
		{
			printf(" , Arrivee %d;%d",(mult->arrivee).x,(mult->arrivee).y);
			mult = mult->multiple;
		}
		printf("\n");
		l = l->suiv;
	}
}
